#!/bin/sh

rm -rf r
rm -rf e
mkdir r
mkdir e

result_path=r
expected_path=e
path=/etc/mysql

###########################################################################################################
#-----------------------Section[2]: Immunization Runtime Environment   ------------------------------------
###########################################################################################################


#-----------------------Section[2-1]:Configuration File Access Rights  ------------------------------------
#Current State:
echo "<------1-1-my.cnf------>" > $result_path/r1-1
ls -lh $path/my.cnf | awk '{print $1,$3,$4}' >> $result_path/r1-1
echo >> $result_path/r1-1 

#Expected State:
echo "<------1-1-my.cnf------>"  > $expected_path/r1-1
echo "-rw-r--r-- root root" >> $expected_path/r1-1
echo  >> $expected_path/r1-1


#-----------------------Section[2-2]:Data storage folder Access Rights ------------------------------------
#Current State:
echo "<------1-2-var/lib/mysql------>" > $result_path/r1-2
if [ ! -d /var/lib/mysql ]; then
    echo "/var/lib/mysql not found!" >> $result_path/r1-2
else
ls -lh -d /var/lib/mysql | awk '{print $1,$3,$4}' >> $result_path/r1-2
fi
echo >> $result_path/r1-2

#Expected State:
echo "<------1-2-var/lib/mysql------>"  > $expected_path/r1-2
echo "drwxrwx--- mysql mysql" >> $expected_path/r1-2
echo  >> $expected_path/r1-2


#-----------------------Section[2-3]:Safe User ------------------------------------------------------------
#Current State:
echo "<------1-3-mysql-user------>" > $result_path/r1-3
grep "MySQL Server" /etc/passwd |awk 'BEGIN { FS=":"; } { print $7; }'  >> $result_path/r1-3
echo >> $result_path/r1-3

#Expected State:
echo "<------1-3-mysql-user------>"  > $expected_path/r1-3
echo "/bin/false" >> $expected_path/r1-3
echo  >> $expected_path/r1-3


#-----------------------Section[2-4]:Log File Access Rights -----------------------------------------------
#Current State:
echo "<------1-4-var/log/mysql------>" > $result_path/r1-4
ls -lh /var/log/mysql* | grep - |awk '{print $1,$3,$4}' >> $result_path/r1-4
echo >> $result_path/r1-4

#Expected State:
echo "<------1-4-var/log/mysql------>"  > $expected_path/r1-4
vv=$(ls -lh /var/log/mysql* | grep - |awk '{print $1}')
for i in $vv; do
echo "-rw-r----- mysql adm" >> $expected_path/r1-4
done
echo  >> $expected_path/r1-4


###########################################################################################################
#-----------------------Section[3]: Secure Connection to The Database -------------------------------------
###########################################################################################################


#-----------------------Section[3-1]:allow-suspicious-udfs Parameter --------------------------------------
#Current State:
echo "<------7-1-allow-suspicious-udfs------>" > $result_path/r7-1
mysqld --verbose --help | grep allow-suspicious-udfs | awk '/allow-suspicious-udfs/{i++}i==2{print $2; exit}' >> $result_path/r7-1
echo >> $result_path/r7-1

#Expected State:
echo "<------7-1-allow-suspicious-udfs------>" > $expected_path/r7-1
echo "FALSE" >> $expected_path/r7-1
echo  >> $expected_path/r7-1


#-----------------------Section[3-2]:create_user_priv Parameter -------------------------------------------
#Current State:
echo "<------7-2-create_user_priv------>" > $result_path/r7-2
MYSQL_PWD=$password mysql --user=$user -e "SELECT user FROM mysql.user WHERE Create_user_priv = 'Y';" | awk '{if ($1 != "user") print $1}' >> $result_path/r7-2
echo >> $result_path/r7-2

#Expected State:
echo "<------7-2-create_user_priv------>" > $expected_path/r7-2
echo $user >> $expected_path/r7-2
echo "debian-sys-maint" >> $expected_path/r7-2
echo >> $expected_path/r7-2


#-----------------------Section[3-3]:secure_file_priv Parameter -------------------------------------------
#Current State:
echo "<------7-3-secure_file_priv------>" > $result_path/r7-3
grep secure-file-priv $path/my.cnf >> $result_path/r7-3
echo >> $result_path/r7-3

#Expected State:
echo "<------7-3-secure_file_priv------>" > $expected_path/r7-3
echo "secure-file-priv = /var/lib/mysql-files" >> $expected_path/r7-3
echo  >> $expected_path/r7-3


#-----------------------Section[3-4]:skip-grant-tables Parameter ------------------------------------------
#Current State:
echo "<------7-4-skip-grant-tables------>" > $result_path/r7-4
grep skip-grant-tables $path/my.cnf >> $result_path/r7-4
echo >> $result_path/r7-4

#Expected State:
echo "<------7-4-skip-grant-tables------>" > $expected_path/r7-4
echo "skip-grant-tables = FALSE" >> $expected_path/r7-4
echo  >> $expected_path/r7-4


#-----------------------Section[3-5]:local_infile Parameter -----------------------------------------------
#Current State:
echo "<------3-1-Local_Infile------>" > $result_path/r3-1
MYSQL_PWD=$password mysql --user=$user -e "SHOW VARIABLES LIKE 'local_infile';" | grep local_infile | awk '{print $2}' >> $result_path/r3-1
echo >> $result_path/r3-1

#Expected State:
echo "<------3-1-Local_Infile------>"  > $expected_path/r3-1
echo "OFF" >> $expected_path/r3-1
echo  >> $expected_path/r3-1


#-----------------------Section[3-6]:skip_show_database Parameter -----------------------------------------
#Current State:
echo "<------3-5-skip_show_database------>" > $result_path/r3-5
MYSQL_PWD=$password mysql --user=$user -e "SHOW VARIABLES LIKE 'skip_show_database';" | grep skip_show_database | awk '{print $2}' >> $result_path/r3-5
echo >> $result_path/r3-5

#Expected State:
echo "<------3-5-skip_show_database------>"  > $expected_path/r3-5
echo "ON" >> $expected_path/r3-5
echo  >> $expected_path/r3-5


#-----------------------Section[3-7]:skip-symbolic-links Parameter ----------------------------------------
#Current State:
echo "<------7-5-skip-symbolic-links------>" > $result_path/r7-5
MYSQL_PWD=123 mysql --user=root -e "SHOW variables LIKE 'have_symlink';"| grep have_symlink | awk '{print $2}' >> $result_path/r7-5
echo >> $result_path/r7-5

#Expected State:
echo "<------7-5-skip-symbolic-links------>" > $expected_path/r7-5
echo "YES" >> $expected_path/r7-5
echo  >> $expected_path/r7-5


#-----------------------Section[3-8]: Delete Test Database ------------------------------------------------
#Current State:
echo "<------3-2-Test_Database------>" > $result_path/r3-2
if [ "a$(MYSQL_PWD=$password mysql --user=$user -e "show databases like 'test';")" = "a" ]
then
    echo "database 'test' does not exist!"  >> $result_path/r3-2
else
    echo "database 'test' exist!"  >> $result_path/r3-2
fi
echo >> $result_path/r3-2

#Expected State:
echo "<------3-2-Test_Database------>"  > $expected_path/r3-2
echo "database 'test' does not exist!" >> $expected_path/r3-2
echo  >> $expected_path/r3-2


#-----------------------Section[3-9]: Delete History File -------------------------------------------------
#Current State:
echo "<------3-3-mysql_history------>" > $result_path/r3-3
ls -lh /root/.mysql_history >/dev/null 2>&1 && var=1 || var=0
if [ $var -eq 1 ] ; then
ls -lh /root/.mysql_history | awk '{print $9,$10,$11}' >> $result_path/r3-3
else
echo "you dont have permission!" >> $result_path/r3-3
fi
echo >> $result_path/r3-3

#Expected State:
echo "<------3-3-mysql_history------>"  > $expected_path/r3-3
echo "/root/.mysql_history -> /dev/null" >> $expected_path/r3-3
echo  >> $expected_path/r3-3


###########################################################################################################
#-----------------------Section[4]: Authentication --------------------------------------------------------
###########################################################################################################

#-----------------------Section[4-1]: bind_address Parameter ----------------------------------------------
#Current State:
echo "<------4-1-bind_address------>" > $result_path/r4-1
grep bind-address $path/my.cnf | awk '{print $3}' >> $result_path/r4-1
echo >> $result_path/r4-1

#Expected State:
echo "<------4-1-bind_address------>"  > $expected_path/r4-1
echo "127.0.0.1" >> $expected_path/r4-1
echo  >> $expected_path/r4-1


#-----------------------Section[4-2]: max_connection Parameter --------------------------------------------
#Current State:
echo "<------4-2-max_connection------>" > $result_path/r4-2
grep max_connection $path/my.cnf | awk '{print $1,$2,$3}' >> $result_path/r4-2
echo >> $result_path/r4-2

#Expected State:
echo "<------4-2-max_connection------>"  > $expected_path/r4-2
echo "max_connections = 100" >> $expected_path/r4-2
echo  >> $expected_path/r4-2


#-----------------------Section[4-4]: max_connect_errors Parameter ----------------------------------------
#Current State:
echo "<------4-3-max_connect_errors------>" > $result_path/r4-3
MYSQL_PWD=$password mysql --user=$user -e "SHOW VARIABLES LIKE 'max_connect_errors';" | grep max_connect_errors | awk '{print $2}' >> $result_path/r4-3
echo >> $result_path/r4-3

#Expected State:
echo "<------4-3-max_connect_errors------>"  > $expected_path/r4-3
echo "5" >> $expected_path/r4-3
echo  >> $expected_path/r4-3


#-----------------------Section[4-5]: port Parameter ------------------------------------------------------
#Current State:
echo "<------4-4-port------>" > $result_path/r4-4
MYSQL_PWD=$password mysql --user=$user -e "SHOW VARIABLES LIKE 'port';" | grep port | awk '{print $2}' >> $result_path/r4-4
echo >> $result_path/r4-4

#Expected State:
echo "<------4-4-port------>"  > $expected_path/r4-4
echo "3307" >> $expected_path/r4-4
echo  >> $expected_path/r4-4


#-----------------------Section[4-6]: OpenSSL Parameter ---------------------------------------------------
#Current State:
echo "<------2-1-OpenSSL------>" > $result_path/r2-1
MYSQL_PWD=$password mysql --user=$user -e "SHOW VARIABLES LIKE '%openssl%';" | grep have_openssl | awk '{print $2}' >> $result_path/r2-1
echo >> $result_path/r2-1

#Expected State:
echo "<------2-1-OpenSSL------>"  > $expected_path/r2-1
echo "YES" >> $expected_path/r2-1
echo  >> $expected_path/r2-1


#-----------------------Section[4-9]: Delete anonymous_account --------------------------------------------
#Current State:
echo "<------6-2-anonymous_account------>" > $result_path/r6-2
ac=$(MYSQL_PWD=$password mysql --user=$user -e "SELECT host,user,authentication_string FROM mysql.user where user='';" | awk 'NR!=1 {print $2}')
if [ -n $ac  ]
then
	echo "no anonymous account"  >> $result_path/r6-2
else
	for i in $ac ; do
	echo $i >> $result_path/r6-2
	done
fi
echo >> $result_path/r6-2

#Expected State:
echo "<------6-2-anonymous_account------>"  > $expected_path/r6-2
if [ -n $ac  ]
then
	echo "no anonymous account"  >> $expected_path/r6-2
else
	for i in $ac ; do
	echo "delete this user!!!" >> $expected_path/r6-2
	done
fi
echo  >> $expected_path/r6-2


#-----------------------Section[4-10]: insecure Account ---------------------------------------------------
#Current State:
echo "<------6-3-unsecure_account------>" > $result_path/r6-3
ua=$(MYSQL_PWD=$password mysql --user=$user -e "SELECT host,user,authentication_string FROM mysql.user where authentication_string='';" | awk 'NR!=1 {print $2}')
if [ -z $ua  ]
then
	echo "no unsecure account"  >> $result_path/r6-3
else
	for i in $ua ; do
	echo $i >> $result_path/r6-3
	done
fi
echo >> $result_path/r6-3

#Expected State:
echo "<------6-3-unsecure_account------>"  > $expected_path/r6-3
if [ -z $ua  ]
then
	echo "no unsecure account"  >> $expected_path/r6-3
else
	for i in $ua ; do
	echo "set password for this user!!!" >> $expected_path/r6-3
	done
fi
echo  >> $expected_path/r6-3


###########################################################################################################
#-----------------------Section[5]: Auditing -------------------------------------------------------------
###########################################################################################################


#-----------------------Section[5-1]: general_log Parameter ----------------------------------------------
#Current State:
echo "<------5-1-general_log------>" > $result_path/r5-1
MYSQL_PWD=$password mysql --user=$user -e "SHOW VARIABLES LIKE 'general_log';" | grep general_log | awk '{print $2}' >> $result_path/r5-1
echo >> $result_path/r5-1

#Expected State:
echo "<------5-1-general_log------>"  > $expected_path/r5-1
echo "ON" >> $expected_path/r5-1
echo  >> $expected_path/r5-1


#-----------------------Section[5-2]: slow_query_log Parameter --------------------------------------------
#Current State:
echo "<------5-2-slow_query_log------>" > $result_path/r5-2
MYSQL_PWD=$password mysql --user=$user -e "SHOW VARIABLES LIKE 'slow_query_log';" | grep slow_query_log | awk '{print $2}' >> $result_path/r5-2
echo >> $result_path/r5-2

#Expected State:
echo "<------5-2-slow_query_log------>"  > $expected_path/r5-2
echo "ON" >> $expected_path/r5-2
echo  >> $expected_path/r5-2


#-----------------------Section[5-3]: log_error Parameter -------------------------------------------------
#Current State:
echo "<------5-3-log_error------>" > $result_path/r5-3
le=$(MYSQL_PWD=$password mysql --user=$user -e "SHOW VARIABLES LIKE 'log_error';" | grep log_error | awk '{print $2}')
if [ -n $le  ]
then
    echo "ON"  >> $result_path/r5-3
else
    echo "OFF"  >> $result_path/r5-3
fi

echo >> $result_path/r5-3

#Expected State:
echo "<------5-3-log_error------>"  > $expected_path/r5-3
echo "ON" >> $expected_path/r5-3
echo  >> $expected_path/r5-3


#-----------------------Section[5-4]: log_bin Parameter ---------------------------------------------------
#Current State:
echo "<------5-2-log_bin------>" > $result_path/r5-4
MYSQL_PWD=$password mysql --user=$user -e "SHOW VARIABLES LIKE 'log_bin';" | grep log_bin | awk '{print $2}' >> $result_path/r5-4
echo >> $result_path/r5-4

#Expected State:
echo "<------5-2-log_bin------>"  > $expected_path/r5-4
echo "ON" >> $expected_path/r5-4
echo  >> $expected_path/r5-4


